S_EMPTY = 0   -- empty state
S_RED = 1     -- red state
S_BLACK = 2   -- black state
EMPTY = 0.05  -- proportion of empty cells
DIM = 50      -- x and y dimensions of the city

-- computes whether an agent of a given 'state' would be satisfied living in a given 'cell'
isSatisfied = function(state, cell)
	count_equal = 0
	count_different = 0
	forEachNeighbor(cell, function(cell, neigh)
		if neigh.state ~= S_EMPTY then
			if neigh.state ~= state then
				count_different = count_different + 1
			else
				count_equal = count_equal + 1
			end
		end
	end)
	return count_equal >= (count_different - 1)
end

citizen = Agent {
	-- a citizen is initialized with a random state (black or red)
	-- with 50% of probability for each
	init = function(self)
		if math.random() < 0.5 then
			self.state = S_RED
		else
			self.state = S_BLACK
		end
	end,
	-- a citizen tries to move to a random empty cell whenever it is
	-- dissatisfied with the current cell it lives
	execute = function(self)
		if not isSatisfied(self.state, self:getCell()) then
			newcell = cs:sample()
			if newcell.state == S_EMPTY and isSatisfied(self.state, newcell) then
				self:getCell().state = S_EMPTY
				self:move(newcell)
				self:getCell().state = self.state
			end
		end
	end
}

soc = Society{
	instance = citizen,
	quantity = DIM * DIM * (1 - EMPTY)
}

cs = CellularSpace {
	xdim = DIM,
	ydim = DIM
}

cs:createNeighborhood{
	strategy = "moore",
	self = false
}

e = Environment {cs, soc}
e:createPlacement{strategy = "random", max = 1}

forEachCell(cs, function(cell)
	cell.state = S_EMPTY
end)

forEachAgent(soc, function(agent)
	agent:getCell().state = agent.state
end)

leg = Legend {
	grouping = "uniquevalue",
	colorBar = {
		{value = S_EMPTY, color = "white"},
		{value = S_BLACK, color = "black"},
		{value = S_RED,   color = "red"}
	}
}

Observer {
	subject = cs,
	attributes = {"state"},
	legends = {leg}
}

t = Timer {
	Event{action = soc},
	Event{action = cs}
}

t:execute(300)

