EMPTY  = 0
BLUE   = 1
RED    = 2
TOLERANCE = 1

function createCellSpace (model)
	model.cellspace = CellularSpace {
		xdim = model.cellSpaceDim,
		ydim = model.cellSpaceDim
	}

    model.cellspace:createNeighborhood {
        strategy = "moore",
  		self = false
	}

	forEachCell (model.cellspace, function (cell)
		cell.state = EMPTY
	end)

end

function satisfied (state,cell)

	num_different = 0
	num_equal = 0 

	forEachNeighbor (cell, function (cell, neigh)
		if neigh.state ~= state and neigh.state ~= EMPTY then
			num_different = num_different + 1
		end
		if neigh.state == state and neigh.state ~= EMPTY then
			num_equal = num_equal + 1
		end
	end)

	if (num_equal >= (num_different - TOLERANCE)) then
		return true
	else
		return false
	end
end

function moveFamily (family)

	--- try to get a place to go 
	newcell = model.cellspace:sample()

	-- is the new cell a good place? Yes, move
	if newcell.state == EMPTY and satisfied (family.state, newcell) then
		family:getCell().state = EMPTY
		family:move (newcell)
		family:getCell().state = family.state
	end
end


function createAgent (model)

	model.family = Agent {
	-- an Agent has two functions (init and execute)
	-- init initializes the agent
	-- execute runs the agent
		-- initialize the agent

		init = function (self)
			if math.random() < 0.5 then
				self.state = BLUE
			else
				self.state = RED
			end
		end,

		execute = function (self)
			if not satisfied (self.state,self:getCell()) then
				moveFamily (self)
			end
		end
	}
end

function createSociety (model)
	model.society = Society {
		instance = model.family, 
		quantity = model.societyDim
	}
end

function createEnvironment (model)

	model.env = Environment {model.cellspace, model.society}

	model.env:createPlacement {
		strategy = "random",
		max = 1 
	}

	forEachAgent (model.society, function (agent)
		agent:getCell().state = agent.state
		end)

end

function createTimer (model)
	model.timer = Timer {
		Event {time = 1, period = 1, action = model.society},
		Event {time = 1, period = 1, action = model.cellspace}
	}
end

function createMap (cellspace)

	leg = Legend {
		grouping = "uniquevalue",
		colorBar = {
			{value = EMPTY, color = "white"},
			{value = BLUE,  color = "blue"},
			{value = RED,   color = "red"}
		}
	}
	map = Observer {
		type = "map",
		subject = cellspace,
		attributes = {"state"},
		legends = {leg}
	}
end

function AgentModel (model)

	-- createCellSpace 
	createCellSpace (model)  -- Ok

	-- create Agent
	createAgent (model)   -- OK

	-- create Society
	createSociety (model) --oK

	-- create Environment 
	createEnvironment (model)

	-- timer 
	createTimer (model)

	-- create Observer
	createMap (model.cellspace)

	return model
end

function run (model, time)
	model.timer:execute (time)
end

model = AgentModel {
	cellSpaceDim  =  50,
	societyDim    =  2400,
}

run (model, 300)