S_EMPTY = 0   -- empty state
S_RED = 1     -- red state
S_BLACK = 2   -- black state

------ SIR BASIC -- version 1 
function createMap (cellspace)

	leg = Legend {
		grouping = "uniquevalue",
		colorBar = {
			{value = S_EMPTY, color = "white"},
			{value = S_BLACK, color = "black"},
			{value = S_RED,  color = "red"}
		}
	}
	map = Observer {
		type = "map",
		subject     = cellspace,
		attributes  = {"state"},
		legends = {leg}
	}
end

----
function createCellSpace(model)

	model.cellspace = CellularSpace {
				xdim = model.cellSpaceDim,
				ydim = model.cellSpaceDim
	}
	model.cellspace:createNeighborhood{
		strategy = "moore",
		self = false
	}

	forEachCell(model.cellspace, function(cell)
			cell.state = S_EMPTY
	end)

	--model.cellspace:notify(0) 
end

--function createAgent (model)

-- computes whether an agent of a given 'state' would be satisfied living in a given 'cell'
function  isSatisfied (state, cell)
	count_equal = 0
	count_different = 0
	forEachNeighbor(cell, function(cell, neigh)
		if neigh.state ~= S_EMPTY then
			if neigh.state ~= state then
				count_different = count_different + 1
			else
				count_equal = count_equal + 1
			end
		end
	end)
	return count_equal >= count_different
end

function createAgent (model)

	model.citizen = Agent {
		-- a citizen is initialized with a random state (black or red)
		-- with 50% of probability for each
		init = function(self)
			if math.random() < 0.5 then
				self.state = S_RED
			else
				self.state = S_BLACK
			end
		end,
	
	-- a citizen tries to move to a random empty cell whenever it is
	-- dissatisfied with the current cell it lives
		execute = function(self)
			if not isSatisfied(self.state, self:getCell()) then
				newcell = model.cellspace:sample()
				if newcell.state == S_EMPTY and isSatisfied(self.state, newcell) then
					self:getCell().state = S_EMPTY
					self:move(newcell)
					self:getCell().state = self.state
				end
			end
	end
	}
end

function createSociety (model)
	model.society = Society{
		instance = model.citizen,
		quantity = model.societyDim
	}
end

function createEnvironment (model)
	model.env = Environment {model.cellspace, model.society}

	model.env:createPlacement{strategy = "random", max = 1}

	forEachAgent(model.society, function(agent)
		agent:getCell().state = agent.state
	end)

end

function createTimer (model)

	model.timer = Timer {
		Event{time = 1, period = 1, action = model.society},
		Event{time = 1, period = 1, action = model.cellspace},
	}
end

function run (model, time)
	model.timer:execute (time)
end

-----------
function AgentModel(model)

	createCellSpace (model)

	-- create Agent
	createAgent (model)

	-- create Society
	createSociety (model)

	-- create Enviroment
	createEnvironment (model)

	-- timer
	createTimer (model)

	-- create Plot
	createMap (model.cellspace)


	return model
end


model = AgentModel {

	cellSpaceDim  =  50,
	societyDim    =  2000,
}

run(model,100)